@@ -34,13 +34,14 @@ from urllib import parse
 from bzt import ManualShutdown, get_configs_dir, TaurusConfigError, TaurusInternalException
 from bzt.utils import reraise, load_class, BetterDict, ensure_is_dict, dehumanize_time, is_windows, is_linux
 from bzt.utils import shell_exec, get_full_path, ExceptionalDownloader, get_uniq_name, HTTPClient, Environment
+from bzt.utils import NETWORK_PROBLEMS
 from .dicts import Configuration
 from .modules import Provisioning, Reporter, Service, Aggregator, EngineModule
 from .names import EXEC, TAURUS_ARTIFACTS_DIR, SETTINGS
 from .templates import Singletone
 from ..environment_helpers import expand_variable_with_os, custom_expandvars, expand_envs_with_os
 
-from bzt.resources.version import VERSION
+from bzt.resources.version import VERSION, DEV_VERSION
 
 
 class Engine(object):
@@ -92,15 +93,17 @@ class Engine(object):
 
         self._http_client = None
 
+        self.graceful_tmp = None
+
     def set_pythonpath(self):
         version = sys.version.split(' ')[0]
         path_suffix = os.path.join('python-packages', version)
         self.user_pythonpath = get_full_path(os.path.join("~", ".bzt", path_suffix))
         self.temp_pythonpath = get_full_path(os.path.join(self.artifacts_dir, path_suffix))
         current_pythonpath = os.environ.get('PYTHONPATH', '')
-        paths = self.user_pythonpath, self.temp_pythonpath, current_pythonpath
+        paths = current_pythonpath, self.temp_pythonpath, self.user_pythonpath
 
-        self.log.debug("Set PYTHONPATH to :\n\tUSER: '{}' +\n\tTEMP: '{}' +\n\tCURRENT: '{}'".format(*paths))
+        self.log.debug("Set PYTHONPATH to :\n\tCURRENT: '{}' +\n\tTEMP: '{}' +\n\tUSER: '{}'".format(*paths))
         try:
             user_packages = os.listdir(self.user_pythonpath)
         except:
@@ -234,7 +237,11 @@ class Engine(object):
         if cwd is None:
             cwd = self.default_cwd
 
-        return shell_exec(args, cwd=cwd, env=env.get(), **kwargs)
+        self.graceful_tmp = self.create_artifact(prefix="GRACEFUL", suffix="")
+        env = env.get()
+        env['GRACEFUL'] = self.graceful_tmp
+        
+        return shell_exec(args, cwd=cwd, env=env, **kwargs)
 
     def run(self):
         """
@@ -308,6 +315,8 @@ class Engine(object):
         """
         self.log.info("Shutting down...")
         self.log.debug("Current stop reason: %s", self.stopping_reason)
+        if self.graceful_tmp and not os.path.exists(self.graceful_tmp):
+            open(self.graceful_tmp, 'x').close()
         exc_info = exc_value = None
         modules = [self.provisioning, self.aggregator] + self.reporters + self.services  # order matters
         for module in modules:
@@ -322,6 +331,8 @@ class Engine(object):
                     exc_value = exc
                     exc_info = sys.exc_info()
 
+        if self.graceful_tmp and os.path.exists(self.graceful_tmp):
+            os.remove(self.graceful_tmp)
         self.config.dump()
         if exc_value:
             reraise(exc_info, exc_value)
@@ -338,6 +349,10 @@ class Engine(object):
         for module in modules:
             if module in self.prepared:
                 try:
+                    if self.stopping_reason and \
+                            not isinstance(self.stopping_reason, KeyboardInterrupt) and \
+                            module.errors is not None:
+                        module.errors.append(str(self.stopping_reason))
                     module.post_process()
                 except BaseException as exc:
                     if isinstance(exc, KeyboardInterrupt):
@@ -422,7 +437,7 @@ class Engine(object):
             os.makedirs(self.artifacts_dir)
 
         # dump current effective configuration
-        dump = self.create_artifact("effective", "")  # TODO: not good since this file not exists
+        dump = self.create_artifact("effective", "")
         self.config.set_dump_file(dump)
         self.config.dump()
 
@@ -706,28 +721,36 @@ class Engine(object):
         return self._http_client
 
     def _check_updates(self, install_id):
+        if VERSION == DEV_VERSION:
+            return
+
+        params = (VERSION, install_id)
+        addr = "https://gettaurus.org/updates/?version=%s&installID=%s" % params
+        self.log.debug("Requesting updates info: %s", addr)
+        client = self.get_http_client()
         try:
-            params = (VERSION, install_id)
-            addr = "https://gettaurus.org/updates/?version=%s&installID=%s" % params
-            self.log.debug("Requesting updates info: %s", addr)
-            client = self.get_http_client()
             response = client.request('GET', addr, timeout=10)
+        except NETWORK_PROBLEMS:
+            self.log.debug("Failed to check for updates: %s", traceback.format_exc())
+            self.log.warning("Failed to check for updates")
+            return
+
+        data = response.json()
+        latest = data.get('latest')
+        needs_upgrade = data.get('needsUpgrade')
+        if latest is None or needs_upgrade is None:
+            self.log.warning(f'Wrong updates info: "{data}"')
+        else:
+            self.log.debug(f'Taurus updates info: "{data}"')
 
-            data = response.json()
-            self.log.debug("Taurus updates info: %s", data)
             mine = LooseVersion(VERSION)
-            latest = LooseVersion(data['latest'])
-            if mine != "DEV" and (mine < latest or data['needsUpgrade']):
+            if (mine < latest) or needs_upgrade:
                 msg = "There is newer version of Taurus %s available, consider upgrading. " \
-                      "What's new: http://gettaurus.org/docs/Changelog/"
+                    "What's new: http://gettaurus.org/docs/Changelog/"
                 self.log.warning(msg, latest)
             else:
                 self.log.debug("Installation is up-to-date")
 
-        except BaseException:
-            self.log.debug("Failed to check for updates: %s", traceback.format_exc())
-            self.log.warning("Failed to check for updates")
-
     def eval_env(self):
         """
         Should be done after `configure`
