@@ -16,7 +16,6 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 import os
-import re
 from distutils.version import LooseVersion
 from lxml import etree
 
@@ -83,6 +82,7 @@ class LoadSettingsProcessor(object):
         self.load = executor.get_specific_load()
         self.raw_load = executor.get_raw_load()
         self.log.debug("Load: %s", self.load)
+        self.force_ctg = executor.settings.get("force-ctg", True)
         self.tg = self._detect_thread_group(executor)
         self.tg_handler = ThreadGroupHandler(self.log)
 
@@ -93,7 +93,7 @@ class LoadSettingsProcessor(object):
         :return:
         """
         tg = self.TG
-        if not executor.settings.get('force-ctg', True):
+        if not self.force_ctg:
             return tg
 
         msg = 'Thread group detection: %s, regular ThreadGroup will be used'
@@ -112,7 +112,7 @@ class LoadSettingsProcessor(object):
 
         return tg
 
-    def modify(self, jmx):
+    def modify(self, jmx, is_jmx_generated=False):
         if not (self.raw_load.iterations or self.raw_load.concurrency or self.load.duration):
             self.log.debug('No iterations/concurrency/duration found, thread group modification is skipped')
             return
@@ -139,8 +139,12 @@ class LoadSettingsProcessor(object):
             target_list = zip(groups, concurrency_list)
 
         for group, concurrency in target_list:
-            self.tg_handler.convert(source=group, target_gtype=self.tg, load=self.load, concurrency=concurrency)
-
+            iterations = None
+            existed_tg = (not is_jmx_generated) and (group.gtype == self.TG)
+            if not self.force_ctg and existed_tg:
+                iterations = group.get_iterations()
+            self.tg_handler.convert(source=group, target_gtype=self.tg, load=self.load,
+                                    concurrency=concurrency, iterations=iterations)
         if self.load.throughput:
             self._add_shaper(jmx)
 
@@ -471,14 +475,7 @@ class JMeterScenarioBuilder(JMX):
         children.extend(self._get_timer(request))
 
         self.__add_assertions(children, request)
-
-        timeout = ProtocolHandler.safe_time(request.priority_option('timeout'))
-        if timeout is not None:
-            children.append(JMX._get_dur_assertion(timeout))
-            children.append(etree.Element("hashTree"))
-
         self.__add_extractors(children, request)
-
         self.__add_jsr_elements(children, request)
 
         return [sampler, children]
@@ -486,7 +483,6 @@ class JMeterScenarioBuilder(JMX):
     def compile_if_block(self, block):
         elements = []
 
-        # TODO: pass jmeter IfController options
         if_controller = JMX._get_if_controller(block.condition)
         then_children = etree.Element("hashTree")
         for compiled in self.compile_requests(block.then_clause):
@@ -624,7 +620,6 @@ class JMeterScenarioBuilder(JMX):
         thread_group = JMX.get_thread_group(testname=self.executor.label)
         thread_group_ht = etree.Element("hashTree", type="tg")
 
-        # NOTE: set realistic dns-cache and JVM prop by default?
         self.request_compiler = RequestCompiler(self)
         for element in self.compile_scenario(self.scenario):
             thread_group_ht.append(element)
@@ -643,7 +638,6 @@ class JMeterScenarioBuilder(JMX):
 
         :type filename: str
         """
-        # NOTE: bad design, as repetitive save will duplicate stuff
         self.__generate()
         super(JMeterScenarioBuilder, self).save(filename)
 
