@@ -30,9 +30,15 @@ from yaml.representer import SafeRepresenter
 
 from bzt import TaurusInternalException, TaurusConfigError
 from bzt.engine import Aggregator
-from bzt.utils import iteritems, dehumanize_time, JSONConvertible
+from bzt.utils import iteritems, dehumanize_time, JSONConvertible, is_int
 
 log = logging.getLogger('aggregator')
+SAMPLE_STATES = 'success', 'jmeter_errors', 'http_errors'
+AGGREGATED_STATES = (
+    '_'.join((SAMPLE_STATES[0], SAMPLE_STATES[1])),     # success_jmeter_errors
+    '_'.join((SAMPLE_STATES[0], SAMPLE_STATES[2])),     # success_http_errors
+    '_'.join((SAMPLE_STATES[2], SAMPLE_STATES[1])),     # http_errors_jmeter_errors
+)
 
 
 class SinglePassIterator(RecordedIterator):
@@ -69,7 +75,7 @@ class SinglePassIterator(RecordedIterator):
         # histogram
         self.hist_values[item.value_iterated_to] = item.count_at_value_iterated_to
 
-        # stddev FIXME: protected mt
+        # stddev
         dev = (self.histogram._hdr_median_equiv_value(item.value_iterated_to) * 1.0) - self._mean * 1000
         self._geometric_dev_total += (dev * dev) * item.count_added_in_this_iter_step
 
@@ -109,7 +115,6 @@ class RespTimesCounter(JSONConvertible):
         new._ff_iterator = self._ff_iterator
         new._perc_levels = self._perc_levels
 
-        # TODO: maybe hdrpy can encapsulate this itself
         new.histogram.counts = copy.deepcopy(self.histogram.counts, memo)
         new.histogram.total_count = self.histogram.total_count
         new.histogram.min_value = self.histogram.min_value
@@ -189,13 +194,16 @@ class KPISet(dict):
     ERRTYPE_ASSERT = 1
     ERRTYPE_SUBSAMPLE = 2
 
-    def __init__(self, perc_levels=(), hist_max_rt=1000.0):
+    def __init__(self, perc_levels=(), hist_max_rt=1000.0, ext_aggregation=False):
         super(KPISet, self).__init__()
         self.sum_rt = 0
         self.sum_lt = 0
         self.sum_cn = 0
         self.perc_levels = perc_levels
-        self._concurrencies = Counter()
+        if ext_aggregation:
+            self.concurrencies = set()
+        else:
+            self.concurrencies = Counter()
         # scalars
         self[KPISet.SAMPLE_COUNT] = 0
         self[KPISet.CONCURRENCY] = 0
@@ -211,14 +219,16 @@ class KPISet(dict):
         self[KPISet.RESP_TIMES] = RespTimesCounter(1, hist_max_rt, 3, perc_levels)
         self[KPISet.RESP_CODES] = Counter()
         self[KPISet.PERCENTILES] = {}
+        self.ext_aggregation = ext_aggregation
 
     def __deepcopy__(self, memo):
         mycopy = KPISet(self.perc_levels, self[KPISet.RESP_TIMES].high)
+        mycopy.ext_aggregation = self.ext_aggregation
         mycopy.sum_rt = self.sum_rt
         mycopy.sum_lt = self.sum_lt
         mycopy.sum_cn = self.sum_cn
         mycopy.perc_levels = self.perc_levels
-        mycopy._concurrencies = copy.deepcopy(self._concurrencies, memo)
+        mycopy.concurrencies = copy.deepcopy(self.concurrencies, memo)
         for key in self:
             mycopy[key] = copy.deepcopy(self.get(key, no_recalc=True), memo)
         return mycopy
@@ -251,10 +261,11 @@ class KPISet(dict):
 
         :type sample: tuple
         """
-        # TODO: introduce a flag to not count failed in resp times? or offer it always?
         cnc, r_time, con_time, latency, r_code, error, trname, byte_count = sample
         self[self.SAMPLE_COUNT] += 1
-        if cnc:
+        if self.ext_aggregation:
+            self.concurrencies.add(trname)
+        elif cnc:
             self.add_concurrency(cnc, trname)
 
         if r_code is not None:
@@ -278,11 +289,16 @@ class KPISet(dict):
 
         if byte_count is not None:
             self[self.BYTE_COUNT] += byte_count
-            # TODO: max/min rt? there is percentiles...
-            # TODO: throughput if interval is not 1s
 
     def add_concurrency(self, cnc, sid):
-        self._concurrencies[sid] = cnc
+        # sid: source id, e.g. node id for jmeter distributed mode
+        if self.ext_aggregation:
+            if isinstance(sid, set):
+                self.concurrencies.update(sid)
+            else:
+                self.concurrencies.add(sid)
+        elif self.concurrencies.get(sid, 0) < cnc:    # take max value of concurrency during the second.
+            self.concurrencies[sid] = cnc
 
     @staticmethod
     def inc_list(values, selector, value):
@@ -338,7 +354,7 @@ class KPISet(dict):
     def values(self):
         raise TaurusInternalException("Invalid call")
 
-    def recalculate(self):  # FIXME: get rid of it at all?
+    def recalculate(self):
         """
         Recalculate averages, stdev and percentiles
 
@@ -349,8 +365,11 @@ class KPISet(dict):
             self[self.AVG_LATENCY] = self.sum_lt / self[self.SAMPLE_COUNT]
             self[self.AVG_RESP_TIME] = self.sum_rt / self[self.SAMPLE_COUNT]
 
-        if len(self._concurrencies):
-            self[self.CONCURRENCY] = sum(self._concurrencies.values())
+        if len(self.concurrencies):
+            if self.ext_aggregation:
+                self[self.CONCURRENCY] = len(self.concurrencies)
+            else:
+                self[self.CONCURRENCY] = sum(self.concurrencies.values())
 
         return self
 
@@ -362,7 +381,7 @@ class KPISet(dict):
         :type src: KPISet
         :return:
         """
-        src.recalculate()  # TODO: could be not resource efficient strat
+        src.recalculate()
 
         self.sum_cn += src.sum_cn
         self.sum_lt += src.sum_lt
@@ -373,6 +392,8 @@ class KPISet(dict):
         self[self.FAILURES] += src[self.FAILURES]
         self[self.BYTE_COUNT] += src[self.BYTE_COUNT]
         # NOTE: should it be average? mind the timestamp gaps
+        if self.ext_aggregation:
+            sid = src.concurrencies
         if src[self.CONCURRENCY]:
             self.add_concurrency(src[self.CONCURRENCY], sid)
 
@@ -380,7 +401,6 @@ class KPISet(dict):
             self[self.RESP_TIMES].merge(src[self.RESP_TIMES])
         elif not self[self.PERCENTILES]:
             # using existing percentiles, in case we have no source data to recalculate them
-            # TODO: it's not valid to overwrite, better take average
             self[self.PERCENTILES] = copy.deepcopy(src[self.PERCENTILES])
 
         self[self.RESP_CODES].update(src[self.RESP_CODES])
@@ -389,13 +409,14 @@ class KPISet(dict):
             self.inc_list(self[self.ERRORS], ('msg', src_item['msg']), src_item)
 
     @staticmethod
-    def from_dict(obj):
+    def from_dict(obj, ext_aggregation=False):
         """
         :type obj: dict
+        :type ext_aggregation: bool
         :rtype: KPISet
         """
         prc_levels = [float(x) for x in obj[KPISet.PERCENTILES].keys()]
-        inst = KPISet(perc_levels=prc_levels)
+        inst = KPISet(perc_levels=prc_levels, ext_aggregation=ext_aggregation)
 
         assert inst.PERCENTILES in obj
 
@@ -504,16 +525,23 @@ class ResultsProvider(object):
         self.cumulative = {}
         self.track_percentiles = [0.0, 50.0, 90.0, 95.0, 99.0, 99.9, 100.0]
         self.listeners = []
-        self.buffer_len = 2
-        self.min_buffer_len = 2
-        self.max_buffer_len = float('inf')
-        self.buffer_multiplier = 2
-        self.buffer_scale_idx = None
+
+        self.buffer_scale_idx = None    # string value of the most interesting percentile (e.q. '90.0')
+        self.buffer_multiplier = 2      # how many sequential samples we want to get before aggregation (approximately)
+
+        self.buffer_len = 2                     # how many data points we want to collect before aggregation
+        self.min_buffer_len = 2                 # small buffer is more responsive but tends to loose data
+        self.max_buffer_len = float('inf')      # buffer_len value can be changed during runtime.
+
         self.histogram_max = 1.0
         self.known_errors = fuzzyset.FuzzySet(use_levenshtein=True)
         self.max_error_count = 100
         self.known_labels = fuzzyset.FuzzySet(use_levenshtein=True)
         self.generalize_labels = 100
+        self._redundant_aggregation = False
+
+    def set_aggregation(self, aggregation):
+        self._redundant_aggregation = aggregation
 
     @staticmethod
     def _fuzzy_fold(key, dataset, limit):
@@ -533,7 +561,7 @@ class ResultsProvider(object):
             return key
 
         size = len(dataset)
-        if size >= limit / 4:  # TODO: parameterize it
+        if size >= limit / 4:
             tolerance = (float(size) / float(limit)) ** 2
             threshold = 1 - tolerance
             matches = dataset.get(key)
@@ -567,7 +595,11 @@ class ResultsProvider(object):
         :param current: KPISet
         """
         for label, data in iteritems(current):
-            cumul = self.cumulative.setdefault(label, KPISet(self.track_percentiles, data[KPISet.RESP_TIMES].high))
+            default = KPISet(
+                perc_levels=self.track_percentiles,
+                hist_max_rt=data[KPISet.RESP_TIMES].high,
+                ext_aggregation=self._redundant_aggregation)
+            cumul = self.cumulative.setdefault(label, default)
             cumul.merge_kpis(data)
             cumul.recalculate()
 
@@ -581,7 +613,7 @@ class ResultsProvider(object):
             current = datapoint[DataPoint.CURRENT]
             if datapoint[DataPoint.CUMULATIVE] or not self._ramp_up_exclude():
                 self.__merge_to_cumulative(current)
-                datapoint[DataPoint.CUMULATIVE] = copy.deepcopy(self.cumulative)  # FIXME: this line eats RAM like hell!
+                datapoint[DataPoint.CUMULATIVE] = copy.deepcopy(self.cumulative)
                 datapoint.recalculate()
 
             for listener in self.listeners:
@@ -608,16 +640,35 @@ class ResultsReader(ResultsProvider):
     Aggregator that reads samples one by one,
     supposed to be attached to every executor
     """
-
     def __init__(self, perc_levels=None):
         super(ResultsReader, self).__init__()
         self.ignored_labels = []
         self.log = logging.getLogger(self.__class__.__name__)
         self.buffer = {}
-        self.min_timestamp = 0
+        self.min_timestamp = 0      # last aggregated timestamp, older data is obsolete and must be fixed
         if perc_levels is not None:
             self.track_percentiles = perc_levels
 
+    @staticmethod
+    def get_mixed_label(label, rc=None, msg=None):
+        # it is used for generation of extended label.
+        # each label data is split according to sample state (success/error/assert)
+
+        def http_mark():
+            http_prefs = ("Embedded resource download error", "Non HTTP response message")
+            return any(msg.startswith(http_pref) for http_pref in http_prefs)
+
+        rc = int(rc) if is_int(rc) else 0  # it's jmeter_error by default
+        if msg:
+            if rc > 299 or http_mark():
+                group = SAMPLE_STATES[2]  # http error
+            else:
+                group = SAMPLE_STATES[1]  # jmeter error
+        else:
+            group = SAMPLE_STATES[0]  # succeeded sample
+
+        return '-'.join((label, str(group)))
+
     def __process_readers(self, final_pass=False):
         """
 
@@ -658,25 +709,46 @@ class ResultsReader(ResultsProvider):
         """
         current = datapoint[DataPoint.CURRENT]
         for sample in samples:
-            label, concur, r_time, con_time, latency, r_code, error, trname, byte_count = sample
-            if label == '':
-                label = '[empty]'
-
-            if self.generalize_labels:
-                label = self._generalize_label(label)
-
-            if label not in current:
-                current[label] = KPISet(self.track_percentiles, self.__get_rtimes_max(label))
+            # sample format: label, conc, r_time, con_time, latency, r_code, error, trname, byte_count
+            base_label = sample[0]
 
             # empty means overall
-            current[label].add_sample((concur, r_time, con_time, latency, r_code, error, trname, byte_count))
+            if base_label == '':
+                base_label = '[empty]'
+
+            if self.generalize_labels:
+                base_label = self._generalize_label(base_label)
+
+            self.__add_sample(current, base_label, sample[1:])
+
+        overall = KPISet(self.track_percentiles, self.__get_rtimes_max(''), ext_aggregation=self._redundant_aggregation)
 
-        overall = KPISet(self.track_percentiles, self.__get_rtimes_max(''))
         for label in current.values():
             overall.merge_kpis(label, datapoint[DataPoint.SOURCE_ID])
         current[''] = overall
+
         return current
 
+    def _get_suffix(self, label):
+        # to collect kpisets to overall sets according to rules result we need to split base label and suffix
+        # the suffixes replace '' label in meaning of 'summary result'
+        if self._redundant_aggregation:
+            return label[label.rfind('-'):]
+        return ''
+
+    def __add_sample(self, current, label, kpis):
+        if self._redundant_aggregation:
+            # kpis format: conc, r_time, con_time, latency, r_code, error_msg, trname, byte_count
+            label = self.get_mixed_label(label=label, rc=kpis[4], msg=kpis[5])
+
+        if label not in current:
+            current[label] = KPISet(
+                perc_levels=self.track_percentiles,
+                hist_max_rt=self.__get_rtimes_max(label),
+                ext_aggregation=self._redundant_aggregation)
+
+        current[label].add_sample(kpis)
+
     def __get_rtimes_max(self, label):
         if label in self.cumulative:
             rtimes_max = self.cumulative[label][KPISet.RESP_TIMES].high
@@ -702,9 +774,12 @@ class ResultsReader(ResultsProvider):
 
         if self.cumulative and self.track_percentiles and self.buffer_scale_idx is not None:
             old_len = self.buffer_len
-            chosen_timing = self.cumulative[''][KPISet.PERCENTILES][self.buffer_scale_idx]
-            self.buffer_len = round(chosen_timing * self.buffer_multiplier)
 
+            # choose average timing of the most interesting percentile
+            chosen_timing = self.cumulative[''][KPISet.PERCENTILES][self.buffer_scale_idx]
+
+            # and calculate new buffer_len based on current speed of data getting
+            self.buffer_len = round(chosen_timing * self.buffer_multiplier)
             self.buffer_len = max(self.min_buffer_len, self.buffer_len)
             self.buffer_len = min(self.max_buffer_len, self.buffer_len)
             if self.buffer_len != old_len:
@@ -748,8 +823,8 @@ class ConsolidatingAggregator(Aggregator, ResultsProvider):
 
     :type underlings: list[bzt.modules.aggregator.ResultsProvider]
     """
+    OVERALL_STATE = "all_transactions_aggregated"
 
-    # TODO: switch to underling-count-based completeness criteria
     def __init__(self):
         Aggregator.__init__(self, is_functional=False)
         ResultsProvider.__init__(self)
@@ -761,6 +836,43 @@ class ConsolidatingAggregator(Aggregator, ResultsProvider):
         self._sticky_concurrencies = {}
         self.min_timestamp = None
 
+    def converter(self, data):
+        if data and self._redundant_aggregation:
+            self.__extend_reported_data(data)
+
+        return data
+
+    def __extend_reported_data(self, kpi_sets):
+        def add_kpi_set_to_state(destination, _state=self.OVERALL_STATE):
+            # add kpi_set to data[<label>][<_state>] value
+            if _state not in destination:
+                destination[_state] = copy.deepcopy(kpi_set)  # avoid merging kpis for first sample
+            else:
+                destination[_state].merge_kpis(kpi_set)  # deepcopy inside
+                destination[_state].recalculate()
+
+        data = kpi_sets['current']
+        overall_label = ''
+        mixed_labels = set(data.keys()) - {overall_label}
+        data[overall_label] = dict()
+        for key in mixed_labels:
+            sep = key.rindex('-')
+            original_label, state = key[:sep], key[sep + 1:]
+            kpi_set = data.pop(key)
+            if original_label not in data:
+                data[original_label] = dict()
+
+            add_kpi_set_to_state(data[overall_label])
+            add_kpi_set_to_state(data[original_label])
+            add_kpi_set_to_state(data[overall_label], state)
+            add_kpi_set_to_state(data[original_label], state)
+
+            for agg_state in AGGREGATED_STATES:
+                if state in agg_state:
+                    add_kpi_set_to_state(data[overall_label], agg_state)
+                    add_kpi_set_to_state(data[original_label], agg_state)
+        pass
+
     def prepare(self):
         """
         Read aggregation options
@@ -772,6 +884,8 @@ class ConsolidatingAggregator(Aggregator, ResultsProvider):
         self.track_percentiles = list(set(self.track_percentiles))
         self.track_percentiles.sort()
         self.settings["percentiles"] = self.track_percentiles
+        
+        self.set_aggregation(self.settings.get('extend-aggregation'))
 
         self.ignored_labels = self.settings.get("ignore-labels", self.ignored_labels)
         self.generalize_labels = self.settings.get("generalize-labels", self.generalize_labels)
@@ -802,6 +916,13 @@ class ConsolidatingAggregator(Aggregator, ResultsProvider):
         self.histogram_max = dehumanize_time(self.settings.get("histogram-initial", self.histogram_max))
         self.max_error_count = self.settings.get("max-error-variety", self.max_error_count)
 
+    def startup(self):
+        super(Aggregator, self).startup()
+
+        # send rules to underlings
+        for underling in self.underlings:
+            underling.set_aggregation(self._redundant_aggregation)
+
     def add_underling(self, underling):
         """
         Add source for aggregating
