@@ -34,15 +34,14 @@ from bzt import AutomatedShutdown, TaurusConfigError, TaurusException, TaurusNet
 from bzt.bza import User, Session, BZA_TEST_DATA_RECEIVED, ENDED
 from bzt.engine import Reporter, Provisioning, Configuration, Service, SETTINGS, ScenarioExecutor, EXEC
 from bzt.modules.aggregator import DataPoint, KPISet, ResultsProvider
-from bzt.modules.console import WidgetProvider, PrioritizedWidget
+from bzt.modules.console import PrioritizedWidget
 from bzt.modules.functional import FunctionalResultsReader, FunctionalSample
 from bzt.modules.monitoring import LocalClient
-from bzt.modules.selenium import SeleniumExecutor
+from bzt.modules._selenium import SeleniumExecutor
 from bzt.modules.services import Unpacker
 from bzt.requests_model import has_variable_pattern
 from bzt.utils import iteritems, open_browser, BetterDict, ExceptionalDownloader, ProgressBarContext
 from bzt.utils import to_json, dehumanize_time, get_full_path, get_files_recursive, replace_in_config
-from bzt.modules.blazemeter.net_utils import get_with_retry
 from bzt.modules.blazemeter.blazemeter_reporter import BlazeMeterUploader
 from bzt.modules.blazemeter.cloud_test import FUNC_API_TEST_TYPE, FUNC_GUI_TEST_TYPE, TAURUS_TEST_TYPE
 from bzt.modules.blazemeter.project_finder import ProjectFinder
@@ -72,9 +71,6 @@ CLOUD_CONFIG_BLACK_LIST = {
         "gatling": {
             "path": True
         },
-        "grinder": {
-            "path": True
-        },
         "junit": {
             "path": True
         },
@@ -124,7 +120,7 @@ class MasterProvisioning(Provisioning):
 
                 if not os.path.exists(self.engine.find_file(rfile)):
                     raise TaurusConfigError("%s: resource file '%s' not found" % (executor, rfile))
-                if to_json(rfile) not in config:  # TODO: might be check is needed to improve
+                if to_json(rfile) not in config:
                     additional_files.append(rfile)
             rfiles += executor_rfiles
 
@@ -183,7 +179,7 @@ class MasterProvisioning(Provisioning):
         return result_list
 
 
-class CloudProvisioning(MasterProvisioning, WidgetProvider):
+class CloudProvisioning(MasterProvisioning):
     """
     :type user: bzt.bza.User
     :type router: BaseCloudTest
@@ -219,9 +215,23 @@ class CloudProvisioning(MasterProvisioning, WidgetProvider):
         bm_settings.update(module.settings)
         module.settings = bm_settings
 
-    def prepare(self):
+    def _validate_passfail(self):
         reporting = self.engine.config.get(Reporter.REP)
+        validate_passfail = any(reporter.get('module') == 'passfail' for reporter in reporting)
+        if validate_passfail:
+            if self.router._test.started_passfail_validation():
+                self.log.info("Please keep in mind that validation can take time.")
+                timeout = 100
+                for i in range(timeout):
+                    if self.router._test.get_passfail_validation():
+                        return
+                    self.log.debug(f"Unsuccessful Passfail validation attempt [{i + 1}]. Retrying...")
+                    sleep(1)
+                self.log.error("Unable get Passfail validation!")
+            else:
+                self.log.error("Unable to validate Passfail configuration!")
 
+    def prepare(self):
         CloudProvisioning.merge_with_blazemeter_config(self)
         CloudProvisioning.configure_client(self)
         self._workspaces = self.user.accounts().workspaces()
@@ -238,9 +248,6 @@ class CloudProvisioning(MasterProvisioning, WidgetProvider):
         is_execution_empty = not self.engine.config.get("execution")
         self.launch_existing_test = self.settings.get("launch-existing-test", is_execution_empty, force_set=True)
 
-        if not self.launch_existing_test:
-            self._filter_reporting()
-
         finder = ProjectFinder(self.parameters, self.settings, self.user, self._workspaces, self.log)
         finder.default_test_name = "Taurus Cloud Test"
 
@@ -268,19 +275,21 @@ class CloudProvisioning(MasterProvisioning, WidgetProvider):
 
             res_files = self.get_rfiles()
             files_for_cloud = self._fix_filenames(res_files)
-
+            self._filter_reporting("blazemeter")    # must not be sent to cloud
             config_for_cloud = self.prepare_cloud_config()
             config_for_cloud.dump(self.engine.create_artifact("cloud", ""))
             del_files = self.settings.get("delete-test-files", True)
             self.router.resolve_test(config_for_cloud, files_for_cloud, del_files)
 
         self.router.sanitize_test()
+        self._validate_passfail()
 
         self.report_name = self.settings.get("report-name", self.report_name)
         if self.report_name == 'ask' and sys.stdin.isatty():
             self.report_name = input("Please enter report-name: ")
 
         self.widget = self.get_widget()
+        self._filter_reporting("passfail")  # must be sent to claud for conversion
 
         if self.engine.is_functional_mode():
             self.results_reader = FunctionalBZAReader(self.log)
@@ -290,22 +299,6 @@ class CloudProvisioning(MasterProvisioning, WidgetProvider):
             self.results_reader.log = self.log
             self.engine.aggregator.add_underling(self.results_reader)
 
-        validate_passfail = any(reporter.get('module') == 'passfail' for reporter in reporting)
-
-        if validate_passfail:
-            if self.router._test.started_passfail_validation():
-                timeout = 100
-                for i in range(timeout):
-                    if self.router._test.get_passfail_validation():
-                        return
-                    self.log.warning(f"Unsuccessful Passfail validation attempt [{i+1}]. Retrying...")
-                    if not i % 10:
-                        self.log.warning("Please keep in mind that validation can take time.")
-                    sleep(1)
-                self.log.error("Unable get Passfail validation!")
-            else:
-                self.log.error("Unable to validate Passfail configuration!")
-
     @staticmethod
     def _get_other_modules(config):
         used_classes = LocalClient.__name__, BlazeMeterUploader.__name__
@@ -353,7 +346,6 @@ class CloudProvisioning(MasterProvisioning, WidgetProvider):
         provisioning = config.get(Provisioning.PROV)
         self._filter_unused_modules(config, provisioning)
 
-        # todo: should we remove config['version'] before sending to cloud?
         config['local-bzt-version'] = config.get('version', 'N/A')
 
         config.filter(CLOUD_CONFIG_BLACK_LIST, black_list=True)
@@ -389,14 +381,14 @@ class CloudProvisioning(MasterProvisioning, WidgetProvider):
             self.log.warning("Dumping available locations instead of running the test:\n%s", table.table)
             raise NormalShutdown("Done listing locations")
 
-    def _filter_reporting(self):
+    def _filter_reporting(self, mod_name):
         reporting = self.engine.config.get(Reporter.REP, [])
         new_reporting = []
         for index, reporter in enumerate(reporting):
             exc = TaurusConfigError("'module' attribute not found in %s" % reporter)
             cls = reporter.get('module', exc)
-            if cls == "blazemeter":
-                self.log.warning("Explicit blazemeter reporting is skipped for cloud")
+            if cls == mod_name:
+                self.log.debug(f"Remove {mod_name} for Cloud provisioning: {reporter}")
             else:
                 new_reporting.append(reporter)
 
@@ -472,7 +464,6 @@ class CloudProvisioning(MasterProvisioning, WidgetProvider):
         self.widget.update()
         return super(CloudProvisioning, self).check()
 
-    @get_with_retry
     def _check_master_status(self):
         return self.router.get_master_status()
 
@@ -702,7 +693,7 @@ class ResultsFromBZA(ResultsProvider):
                 error=msg,
                 ret_c=errors[msg]['rc'],
                 cnt=errors[msg]['count'],
-                errtype=KPISet.ERRTYPE_ERROR,  # TODO: what about asserts?
+                errtype=KPISet.ERRTYPE_ERROR,
                 urls=Counter(), tag=None)
             result.append(kpi_error)
         return result
