@@ -20,7 +20,6 @@ import re
 import sys
 import time
 import traceback
-from abc import abstractmethod
 from collections import deque
 from datetime import datetime
 from itertools import groupby, islice, chain
@@ -37,7 +36,7 @@ from urwid.graphics import BigText
 from urwid.listbox import SimpleListWalker
 from urwid.widget import Divider
 
-from bzt.engine import Reporter, Singletone
+from bzt.engine import Reporter, Singletone, ScenarioExecutor
 from bzt.modules.aggregator import DataPoint, KPISet, AggregatorListener, ResultsProvider
 from bzt.modules.provisioning import Local
 from bzt.utils import humanize_time, is_windows, DummyScreen
@@ -59,8 +58,6 @@ class ConsoleStatusReporter(Reporter, AggregatorListener, Singletone):
     Class to show process status on the console
     :type logger_handlers: list[StreamHandler]
     """
-
-    # NOTE: maybe should use separate thread for screen re-painting
     def __init__(self):
         super(ConsoleStatusReporter, self).__init__()
         self._last_datapoint = None
@@ -131,11 +128,12 @@ class ConsoleStatusReporter(Reporter, AggregatorListener, Singletone):
         if isinstance(self.engine.provisioning, Local):
             modules += self.engine.provisioning.executors
         for module in modules:
-            if isinstance(module, WidgetProvider):
+            if isinstance(module, ScenarioExecutor):
                 widget = module.get_widget()
-                widgets.append(widget)
-                if isinstance(widget, ExecutorWidget):
-                    self.executor_widgets.append(widget)
+                if widget:
+                    widgets.append(widget)
+                    if isinstance(widget, ExecutorWidget):
+                        self.executor_widgets.append(widget)
 
         self.console = TaurusConsole(widgets)
         self.screen.register_palette(self.console.palette)
@@ -1116,21 +1114,6 @@ class TaurusLogo(Pile):
         self._invalidate()
 
 
-class WidgetProvider(object):
-    """
-    Mixin for classes that provide executor widgets
-    """
-
-    @abstractmethod
-    def get_widget(self):
-        """
-        Returns widget instance to be added to sidebar
-
-        :rtype: urwid.Widget
-        """
-        pass
-
-
 class PrioritizedWidget(object):
     def __init__(self, priority=0):
         self.priority = priority
