@@ -16,20 +16,18 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 import codecs
-import json
 import os
 import re
 import time
 from collections import defaultdict
-from distutils.version import LooseVersion
 
 from bzt import TaurusConfigError, ToolError
-from bzt.engine import ScenarioExecutor, Scenario, FileLister, HavingInstallableTools, SelfDiagnosable
+from bzt.engine import ScenarioExecutor, Scenario
 from bzt.modules.aggregator import ConsolidatingAggregator, ResultsReader
-from bzt.modules.console import WidgetProvider, ExecutorWidget
-from bzt.requests_model import HTTPRequest
+from bzt.modules.console import ExecutorWidget
+from bzt.requests_model import HTTPRequest, SetVariables, HierarchicRequestParser
 from bzt.utils import TclLibrary, EXE_SUFFIX, dehumanize_time, get_full_path, FileReader, RESOURCES_DIR, BetterDict
-from bzt.utils import simple_body_dict, CALL_PROBLEMS, numeric_types
+from bzt.utils import CALL_PROBLEMS, convert_body_to_string
 from bzt.utils import unzip, RequiredTool, JavaVM, shutdown_process, ensure_is_dict, is_windows
 
 
@@ -79,7 +77,19 @@ class GatlingScriptBuilder(object):
 
     def _get_exec(self):
         exec_str = ''
-        for req in self.scenario.get_requests():
+        for req in self.scenario.get_requests(parser=HierarchicRequestParser):
+            if isinstance(req, SetVariables):
+                if len(exec_str) > 0:
+                    exec_str += '.'
+
+                exec_str += "exec(\n"
+                exec_str += self.indent("_", level=2)
+                for k, v in sorted(req.mapping.items()):
+                    exec_str += '.set("%s", "%s")' % (k.replace("\"", "\\\""), v.replace("\"", "\\\""))
+
+                exec_str += "\n" + self.indent(")", level=1)
+                continue
+
             if not isinstance(req, HTTPRequest):
                 msg = "Gatling simulation generator doesn't support '%s' blocks, skipping"
                 self.log.warning(msg, req.NAME)
@@ -102,21 +112,14 @@ class GatlingScriptBuilder(object):
                 exec_template = self.indent('.header("%(key)s", "%(val)s")\n', level=3)
                 exec_str += exec_template % {'key': key, 'val': req.headers[key]}
 
-            # todo: join with the same in get_sampler_pair
-            if isinstance(req.body, (dict, list, numeric_types)):
-                if req.get_header('content-type') == 'application/json' or isinstance(req.body, numeric_types):
-                    req.body = json.dumps(req.body)
-                elif not simple_body_dict(req.body):
-                    self.log.debug('Header "Content-Type: application/json" is required for body: "%s"', req.body)
-                    req.body = json.dumps(req.body)
-
+            convert_body_to_string(req)
             if isinstance(req.body, str):
-                exec_str += self.indent('.body(%(method)s("""%(body)s"""))\n', level=3)
-                exec_str = exec_str % {'method': 'StringBody', 'body': req.body}
+                stmt = '.body(%(method)s("""%(body)s"""))\n' % {'method': 'StringBody', 'body': req.body}
+                exec_str += self.indent(stmt, level=3)
             elif isinstance(req.body, dict):
                 for key in sorted(req.body.keys()):
-                    exec_str += self.indent('.formParam("%(key)s", "%(val)s")\n', level=3)
-                    exec_str = exec_str % {'key': key, 'val': req.body[key]}
+                    stmt = '.formParam("%(key)s", "%(val)s")\n' % {'key': key, 'val': req.body[key]}
+                    exec_str += self.indent(stmt, level=3)
             elif req.body is not None:
                 self.log.warning("Unknown body type: %s", req.body)
 
@@ -247,7 +250,7 @@ class GatlingScriptBuilder(object):
         return template_line % params
 
 
-class GatlingExecutor(ScenarioExecutor, WidgetProvider, FileLister, HavingInstallableTools, SelfDiagnosable):
+class GatlingExecutor(ScenarioExecutor):
     """
     Gatling executor module
     """
@@ -699,7 +702,7 @@ class Gatling(RequiredTool):
     """
     DOWNLOAD_LINK = "https://repo1.maven.org/maven2/io/gatling/highcharts/gatling-charts-highcharts-bundle" \
                     "/{version}/gatling-charts-highcharts-bundle-{version}-bundle.zip"
-    VERSION = "3.5.1"
+    VERSION = "3.7.5"
     LOCAL_PATH = "~/.bzt/gatling-taurus/{version}/bin/gatling{suffix}"
 
     def __init__(self, config=None, **kwargs):
