@@ -17,13 +17,12 @@ import os
 from abc import abstractmethod
 
 from bzt import TaurusConfigError
-from bzt.engine import HavingInstallableTools
 from bzt.modules import SubprocessedExecutor
 from bzt.utils import TclLibrary, RequiredTool, Node, CALL_PROBLEMS, RESOURCES_DIR
 from bzt.utils import get_full_path, is_windows, to_json, dehumanize_time, iteritems
 
 
-class JavaScriptExecutor(SubprocessedExecutor, HavingInstallableTools):
+class JavaScriptExecutor(SubprocessedExecutor):
     def __init__(self):
         super(JavaScriptExecutor, self).__init__()
         self.tools_dir = None
@@ -227,11 +226,6 @@ class NewmanExecutor(JavaScriptExecutor):
         if load.iterations:
             cmdline += ['--iteration-count', str(load.iterations)]
 
-        # TODO: allow running several collections like directory, see https://github.com/postmanlabs/newman/issues/871
-        # TODO: support hold-for, probably by having own runner
-        # if load.hold:
-        #    cmdline += ['--hold-for', str(load.hold)]
-
         self.process = self._execute(cmdline, cwd=script_dir)
 
     def _dump_vars(self, key):
@@ -290,7 +284,7 @@ class NPMPackage(RequiredTool):
 
     def __init__(self, tools_dir, node_tool, npm_tool, **kwargs):
         super(NPMPackage, self).__init__(**kwargs)
-        self.package_name = self.PACKAGE_NAME   # todo: split package_name in the constants block
+        self.package_name = self.PACKAGE_NAME
         if self.package_name.startswith("@"):
             package_name_split = self.package_name.split("@")
             self.package_name = '@{}'.format(package_name_split[1])
@@ -336,23 +330,23 @@ class NPMPackage(RequiredTool):
 
 
 class Mocha(NPMPackage):
-    PACKAGE_NAME = "mocha@4.0.1"
+    PACKAGE_NAME = "mocha@9.2.1"
 
 
 class JSSeleniumWebdriver(NPMPackage):
-    PACKAGE_NAME = "selenium-webdriver@3.6.0"
+    PACKAGE_NAME = "selenium-webdriver@4.1.1"
 
 class WDIO(NPMPackage):
-    PACKAGE_NAME = "@wdio/cli@6.1.9"
+    PACKAGE_NAME = "@wdio/cli@7.16.15"
 
 class WDIORunner(NPMPackage):
-    PACKAGE_NAME = "@wdio/local-runner@6.1.9"
+    PACKAGE_NAME = "@wdio/local-runner@7.16.15"
 
 class WDIOReporter(NPMPackage):
-    PACKAGE_NAME = "@wdio/reporter@6.1.9"
+    PACKAGE_NAME = "@wdio/reporter@7.16.14"
 
 class WDIOMochaPlugin(NPMPackage):
-    PACKAGE_NAME = "@wdio/mocha-framework@6.1.8"
+    PACKAGE_NAME = "@wdio/mocha-framework@7.16.15"
 
 
 class Newman(NPMPackage):
