@@ -34,11 +34,11 @@ from cssselect import GenericTranslator
 from lxml import etree
 
 from bzt import TaurusConfigError, ToolError, TaurusInternalException, TaurusNetworkError
-from bzt.engine import Scenario, FileLister, HavingInstallableTools, ScenarioExecutor
-from bzt.engine import SelfDiagnosable, SETTINGS
+from bzt.engine import Scenario, ScenarioExecutor
+from bzt.engine import SETTINGS
 from bzt.jmx import JMX, JMeterScenarioBuilder, LoadSettingsProcessor, try_convert
 from bzt.modules.aggregator import ResultsReader, DataPoint, KPISet
-from bzt.modules.console import WidgetProvider, ExecutorWidget
+from bzt.modules.console import ExecutorWidget
 from bzt.modules.functional import FunctionalResultsReader, FunctionalSample
 from bzt.requests_model import ResourceFilesCollector, has_variable_pattern, HierarchicRequestParser
 from bzt.utils import iteritems, numeric_types, unicode_decode
@@ -80,7 +80,7 @@ def parse_assertion(element, default=""):
     return failure_message, name
 
 
-class JMeterExecutor(ScenarioExecutor, WidgetProvider, FileLister, HavingInstallableTools, SelfDiagnosable):
+class JMeterExecutor(ScenarioExecutor):
     """
     JMeter executor module
 
@@ -94,7 +94,7 @@ class JMeterExecutor(ScenarioExecutor, WidgetProvider, FileLister, HavingInstall
     def __init__(self):
         super(JMeterExecutor, self).__init__()
         self.original_jmx = None
-        self.modified_jmx = None
+        self.modified_jmx = ""
         self.jmeter_log = None
         self.properties = BetterDict()
         self.properties_file = None
@@ -333,6 +333,10 @@ class JMeterExecutor(ScenarioExecutor, WidgetProvider, FileLister, HavingInstall
                 tool_dir = get_full_path(self.tool.tool_path, step_up=2)
             self.env.set({"JMETER_HOME": tool_dir})
 
+        user_cmd = self.settings.get("cmdline")
+        if user_cmd:
+            cmdline += user_cmd.split(" ")
+
         self.process = self._execute(cmdline)
 
     def check(self):
@@ -557,7 +561,7 @@ class JMeterExecutor(ScenarioExecutor, WidgetProvider, FileLister, HavingInstall
         self.__fill_empty_delimiters(jmx)
 
         self.__apply_modifications(jmx)
-        LoadSettingsProcessor(self).modify(jmx)
+        LoadSettingsProcessor(self).modify(jmx, is_jmx_generated)
 
         return jmx
 
@@ -751,7 +755,7 @@ class JMeterExecutor(ScenarioExecutor, WidgetProvider, FileLister, HavingInstall
                 parts = path.split('>')
                 if len(parts) < 2:
                     raise TaurusConfigError("JMeter: property selector must have at least 2 levels")
-                sel_parts = ["[testname='%s']" % parts[0]]  # TODO: support wildcards in element names
+                sel_parts = ["[testname='%s']" % parts[0]]
 
                 for add in parts[1:]:
                     sel_parts.append("[name='%s']" % add)
@@ -831,10 +835,16 @@ class JTLReader(ResultsReader):
         self.csvreader = IncrementalCSVReader(self.log, filename)
         self.read_records = 0
         if errors_filename:
-            self.errors_reader = JTLErrorsReader(errors_filename, parent_logger, err_msg_separator)
+            self.errors_reader = JTLErrorsReader(
+                errors_filename, parent_logger, err_msg_separator, label_converter=self.get_mixed_label)
         else:
             self.errors_reader = None
 
+    def set_aggregation(self, aggregation):
+        super().set_aggregation(aggregation)
+        if self.errors_reader:
+            self.errors_reader.set_aggregation(aggregation)
+
     def _read(self, last_pass=False):
         """
         Generator method that returns next portion of data
@@ -851,7 +861,7 @@ class JTLReader(ResultsReader):
                 trname = row["Hostname"] + row["threadName"][:row["threadName"].rfind('-')]
             else:
                 concur = int(row["allThreads"])
-                trname = ''
+                trname = row["threadName"] if self._redundant_aggregation else ""
 
             rtm = int(row["elapsed"]) / 1000.0
             ltc = int(row["Latency"]) / 1000.0
@@ -865,7 +875,7 @@ class JTLReader(ResultsReader):
                 rcd = rcd.split('.')[-1]
 
             if row["success"] != "true":
-                error = row["responseMessage"]
+                error = row["responseMessage"] if row["responseMessage"] else 'unknown error'
             else:
                 error = None
 
@@ -878,7 +888,6 @@ class JTLReader(ResultsReader):
     def _calculate_datapoints(self, final_pass=False):
         for point in super(JTLReader, self)._calculate_datapoints(final_pass):
             if self.errors_reader:
-                self.errors_reader.read_file()
                 err_details = self.errors_reader.get_data(point[DataPoint.TIMESTAMP])  # get only for labels we have
                 for label in err_details:
                     if label in point[DataPoint.CURRENT]:
@@ -887,6 +896,9 @@ class JTLReader(ResultsReader):
                         self.log.warning("Had error data but no KPISet %s: %s", label, err_details[label])
 
                 for label, label_data in iteritems(point[DataPoint.CURRENT]):
+                    if self._redundant_aggregation:
+                        continue
+
                     if label in err_details:
                         pass
                     elif label_data[KPISet.ERRORS]:
@@ -1140,7 +1152,7 @@ class IncrementalCSVReader(object):
             bytes_read += len(line)
 
             if self.csv_reader is None:
-                dialect = guess_csv_dialect(line, force_doublequote=True)  # TODO: configurable doublequoting?
+                dialect = guess_csv_dialect(line, force_doublequote=True)
                 self.csv_reader = csv.DictReader(self.buffer, [], dialect=dialect)
                 self.csv_reader.fieldnames += line.strip().split(self.csv_reader.dialect.delimiter)
                 self.log.debug("Analyzed header line: %s", self.csv_reader.fieldnames)
@@ -1175,7 +1187,7 @@ class JTLErrorsReader(object):
     """
     url_xpath = GenericTranslator().css_to_xpath("java\\.net\\.URL")
 
-    def __init__(self, filename, parent_logger, err_msg_separator=None):
+    def __init__(self, filename, parent_logger, err_msg_separator=None, label_converter=None):
         # http://stackoverflow.com/questions/9809469/python-sax-to-lxml-for-80gb-xml/9814580#9814580
         super(JTLErrorsReader, self).__init__()
         self.log = parent_logger.getChild(self.__class__.__name__)
@@ -1184,6 +1196,11 @@ class JTLErrorsReader(object):
         self.buffer = BetterDict()
         self.failed_processing = False
         self.err_msg_separator = err_msg_separator
+        self.label_converter = label_converter
+        self._redundant_aggregation = False
+
+    def set_aggregation(self, aggregation):
+        self._redundant_aggregation = aggregation
 
     def read_file(self, final_pass=False):
         """
@@ -1270,11 +1287,13 @@ class JTLErrorsReader(object):
 
         err_item = KPISet.error_item_skel(f_msg, f_rc, 1, f_type, url_counts, f_tag)
         buf = self.buffer.get(t_stamp, force_set=True)
+        if self._redundant_aggregation:
+            label = self.label_converter(label=label, rc=r_code, msg=r_msg)
         KPISet.inc_list(buf.get(label, [], force_set=True), ("msg", f_msg), err_item)
         KPISet.inc_list(buf.get('', [], force_set=True), ("msg", f_msg), err_item)
 
     def _extract_nonstandard(self, elem):
-        t_stamp = int(elem.findtext("timeStamp")) / 1000.0  # NOTE: will it be sometimes EndTime?
+        t_stamp = int(elem.findtext("timeStamp")) / 1000.0
         label = elem.findtext("label")
         message = elem.findtext("responseMessage")
         r_code = elem.findtext("responseCode")
@@ -1352,15 +1371,15 @@ class JMeter(RequiredTool):
     """
     JMeter tool
     """
-    PLUGINS_MANAGER_VERSION = "1.3"
-    PLUGINS_MANAGER = 'https://search.maven.org/remotecontent?filepath=kg/apc/jmeter-plugins-manager/{version}/jmeter-plugins-manager-{version}.jar'
+    PLUGINS_MANAGER_VERSION = "1.7"
+    PLUGINS_MANAGER_LINK = 'https://search.maven.org/remotecontent?filepath=kg/apc/jmeter-plugins-manager/{version}/jmeter-plugins-manager-{version}.jar'
     COMMAND_RUNNER_VERSION = "2.2"
-    COMMAND_RUNNER = 'https://search.maven.org/remotecontent?filepath=kg/apc/cmdrunner/{version}/cmdrunner-{version}.jar'
-    VERSION = "5.2.1"
+    COMMAND_RUNNER_LINK = 'https://search.maven.org/remotecontent?filepath=kg/apc/cmdrunner/{version}/cmdrunner-{version}.jar'
+    VERSION = "5.4.3"
 
     def __init__(self, config=None, props=None, **kwargs):
-        settings = config or {}
-        props = props or {}
+        settings = config or BetterDict()
+        props = props or BetterDict()
 
         version = settings.get("version", JMeter.VERSION)
         jmeter_path = settings.get("path", "~/.bzt/jmeter-taurus/{version}/")
@@ -1370,18 +1389,20 @@ class JMeter(RequiredTool):
         if download_link is not None:
             download_link = download_link.format(version=version)
 
-        plugins_manager_settings = settings.get("plugins-manager", {})
+        plugins_manager_settings = settings.get("plugins-manager")
         plugins_manager_version = plugins_manager_settings.get("version", JMeter.PLUGINS_MANAGER_VERSION)
-        self.plugins_manager = plugins_manager_settings.get("download-link", JMeter.PLUGINS_MANAGER).format(version=plugins_manager_version)
+        self.plugins_manager_link = plugins_manager_settings.get(
+            "download-link", JMeter.PLUGINS_MANAGER_LINK).format(version=plugins_manager_version)
 
-        command_runner_settings = settings.get("command-runner", {})
+        command_runner_settings = settings.get("command-runner")
         command_runner_version = command_runner_settings.get("version", JMeter.COMMAND_RUNNER_VERSION)
-        self.command_runner = command_runner_settings.get("download-link", JMeter.COMMAND_RUNNER).format(version=command_runner_version)
-
+        self.command_runner_link = command_runner_settings.get(
+            "download-link", JMeter.COMMAND_RUNNER_LINK).format(version=command_runner_version)
         self.plugins = settings.get("plugins", [])
 
         super(JMeter, self).__init__(tool_path=jmeter_path, download_link=download_link, version=version, **kwargs)
 
+        self.fix_jars = settings.get("fix-jars", True)
         self.mirror_manager = JMeterMirrorsManager(self.http_client, self.log, self.version)
 
         additional_jvm_props = self._get_jvm_props(props)
@@ -1494,9 +1515,9 @@ class JMeter(RequiredTool):
                     self.log.debug("Error details: %s", traceback.format_exc())
                     raise TaurusNetworkError("Error while downloading %s: %s" % (_file, exc))
 
-    def __install_plugins_manager(self, plugins_manager_path):
+    def __install_plugins_manager(self, pm_installer_path):
         installer = "org.jmeterplugins.repository.PluginManagerCMDInstaller"
-        cmd_line = ["java", "-cp", plugins_manager_path, installer]
+        cmd_line = ["java", "-cp", pm_installer_path, installer]
         self.log.debug("Trying: %s", cmd_line)
         try:
             out, err = self.call(cmd_line)
@@ -1505,7 +1526,8 @@ class JMeter(RequiredTool):
 
         self.log.debug("Install PluginsManager: %s / %s", out, err)
 
-    def __install_plugins(self, plugins_manager_cmd):
+    def __install_plugins(self):
+        plugins_manager_cmd = self._pmgr_path()
         plugin_str = ",".join(self.plugins)
         self.log.info("Installing JMeter plugins: %s", plugin_str)
         cmd_line = [plugins_manager_cmd, 'install', plugin_str]
@@ -1525,25 +1547,65 @@ class JMeter(RequiredTool):
         dest = get_full_path(self.tool_path, step_up=2)
         return os.path.join(dest, 'bin', 'PluginsManagerCMD' + EXE_SUFFIX)
 
+    def _get_jar_fixes(self, lib_dir):
+        direct_install_tools = []
+        if self.fix_jars:
+            # these jars should be replaced with newer version in order to fix some vulnerabilities
+
+            # component name and download link in https://repo1.maven.org/maven2/
+            affected_components = {
+                # Needs to be <1.4.18 for now https://stackoverflow.com/questions/30812293/com-thoughtworks-xstream-security-forbiddenclassexception
+                "xstream": "com/thoughtworks/xstream/xstream/1.4.17/xstream-1.4.17.jar",
+                "jackson-annotations": "com/fasterxml/jackson/core/jackson-annotations/2.13.3/jackson-annotations-2.13.3.jar",
+                "jackson-core": "com/fasterxml/jackson/core/jackson-core/2.13.3/jackson-core-2.13.3.jar",
+                "jackson-databind": "com/fasterxml/jackson/core/jackson-databind/2.13.3/jackson-databind-2.13.3.jar",
+                "netty-codec": "io/netty/netty-codec/4.1.77.Final/netty-codec-4.1.77.Final.jar",
+                "jetty-io": "org/eclipse/jetty/jetty-io/9.4.46.v20220331/jetty-io-9.4.46.v20220331.jar",
+                "snakeyaml": "org/yaml/snakeyaml/1.30/snakeyaml-1.30.jar",
+                "json-smart": "net/minidev/json-smart/2.4.8/json-smart-2.4.8.jar",
+                "xmlgraphics-commons": "org/apache/xmlgraphics/xmlgraphics-commons/2.7/xmlgraphics-commons-2.7.jar"}
+
+            if LooseVersion(self.version) <= LooseVersion('5.4.2'):  # log4j must be fixed till jmeter 5.4.2
+                affected_names = ["log4j-core", "log4j-api", "log4j-slf4j-impl", "log4j-1.2-api"]
+                fixed_version = '2.17.2'
+                maven_link = "org/apache/logging/log4j/{name}/{ver}/{name}-{ver}.jar"
+
+                for name in affected_names:
+                    affected_components[name] = maven_link.format(name=name, ver=fixed_version)
+
+            jar_files = [_file for _file in os.listdir(lib_dir) if _file.endswith(".jar")]
+            for jar_file in jar_files:
+                for comp_name in affected_components:
+                    if jar_file.startswith(comp_name):
+                        download_link = "https://repo1.maven.org/maven2/" + affected_components[comp_name]
+                        target_path = os.path.join(lib_dir, jar_file)
+                        direct_install_tools.append([download_link, target_path])
+                        break
+
+        return direct_install_tools
+
     def install(self):
-        dest = get_full_path(self.tool_path, step_up=2)
-        self.log.info("Will install %s into %s", self.tool_name, dest)
-        plugins_manager_name = os.path.basename(self.plugins_manager)
-        command_runner_name = os.path.basename(self.command_runner)
-        plugins_manager_path = os.path.join(dest, 'lib', 'ext', plugins_manager_name)
-        command_runner_path = os.path.join(dest, 'lib', command_runner_name)
-        direct_install_tools = [  # source link and destination
-            [self.plugins_manager, plugins_manager_path],
-            [self.command_runner, command_runner_path]]
-        plugins_manager_cmd = self._pmgr_path()
+        jmeter_dir = get_full_path(self.tool_path, step_up=2)
+        lib_dir = os.path.join(jmeter_dir, 'lib')
+        self.log.info("Will install %s into %s", self.tool_name, jmeter_dir)
+        plugins_manager_name = os.path.basename(self.plugins_manager_link)
+        command_runner_name = os.path.basename(self.command_runner_link)
+        pm_installer_path = os.path.join(lib_dir, 'ext', plugins_manager_name)
+        command_runner_path = os.path.join(lib_dir, command_runner_name)
+
+        self.__install_jmeter(jmeter_dir)
+
+        direct_install_tools = self._get_jar_fixes(lib_dir)  # format: download link, local destination path
+        direct_install_tools.extend([
+            [self.plugins_manager_link, pm_installer_path],
+            [self.command_runner_link, command_runner_path]])
 
-        self.__install_jmeter(dest)
         self.__download_additions(direct_install_tools)
-        self.__install_plugins_manager(plugins_manager_path)
-        self.__install_plugins(plugins_manager_cmd)
+        self.__install_plugins_manager(pm_installer_path)
+        self.__install_plugins()
 
         cleaner = JarCleaner(self.log)
-        cleaner.clean(os.path.join(dest, 'lib'))
+        cleaner.clean(lib_dir)
 
     def ctg_plugin_installed(self):
         """
