@@ -24,14 +24,14 @@ from urllib import parse
 from shutil import which
 
 from bzt import TaurusConfigError, ToolError, TaurusInternalException
-from bzt.engine import FileLister, ScenarioExecutor, HavingInstallableTools, SelfDiagnosable
+from bzt.engine import ScenarioExecutor
 from bzt.modules.aggregator import ConsolidatingAggregator, ResultsReader
-from bzt.modules.console import WidgetProvider, ExecutorWidget
+from bzt.modules.console import ExecutorWidget
 from bzt.requests_model import HTTPRequest
 from bzt.utils import CALL_PROBLEMS, shutdown_process, RequiredTool, dehumanize_time, FileReader, etree, iteritems
 
 
-class TsungExecutor(ScenarioExecutor, WidgetProvider, FileLister, HavingInstallableTools, SelfDiagnosable):
+class TsungExecutor(ScenarioExecutor):
     """
     Tsung executor module
     """
@@ -101,8 +101,14 @@ class TsungExecutor(ScenarioExecutor, WidgetProvider, FileLister, HavingInstalla
             '-l', self.tsung_artifacts_basedir,
             '-i', self.tsung_controller_id,
             '-w', '0',
-            'start',
         ]
+
+        user_cmd = self.settings.get("cmdline")
+        if user_cmd:
+            args += user_cmd.split(" ")
+
+        args.append('start')
+
         self.process = self._execute(args)
 
     def check(self):
@@ -123,7 +129,7 @@ class TsungExecutor(ScenarioExecutor, WidgetProvider, FileLister, HavingInstalla
 
     def get_widget(self):
         if not self.widget:
-            self.widget = ExecutorWidget(self, self.tsung_controller_id)
+            self.widget = ExecutorWidget(self, "Tsung: " + self.tsung_controller_id)
         return self.widget
 
     def resource_files(self):
@@ -296,7 +302,6 @@ class TsungConfig(object):
             return time_amount, "second"
 
     def __gen_clients(self):
-        # TODO: distributed clients?
         clients = etree.Element("clients")
         client = etree.Element("client", host="localhost", use_controller_vm="true")
         clients.append(client)
