@@ -6,7 +6,7 @@ import sys
 import time
 from collections import OrderedDict
 
-from locust import main, events, runners
+from locust import main, events
 from locust.exception import StopUser
 from requests.exceptions import HTTPError
 
@@ -63,8 +63,6 @@ class LocustStarter(object):
             ('responseCode', rcode),
             ('responseMessage', rmsg),
             ('success', 'true' if exc is None else 'false'),
-
-            # NOTE: might be resource-consuming
             ('Latency', 0),
         ])
 
@@ -72,23 +70,23 @@ class LocustStarter(object):
         if 'runner' in args:
             self.runner = args['runner']
 
-    def __on_request_success(self, request_type, name, response_time, response_length):
+    def __on_request_success(self, request_type, name, response_time, response_length, **args):
         self.num_requests -= 1
         self.writer.writerow(self.__getrec(request_type, name, response_time, response_length))
         self.fhd.flush()
         self.__check_limits()
 
-    def __on_request_failure(self, request_type, name, response_time, exception, response_length=0):
+    def __on_request_failure(self, request_type, name, response_time, exception, response_length=0, **args):
         self.num_requests -= 1
         self.writer.writerow(self.__getrec(request_type, name, response_time, response_length, exception))
         self.fhd.flush()
         self.__check_limits()
 
-    def __on_exception(self, locust_instance, exception, tb):
+    def __on_exception(self, locust_instance, exception, tb, **args):
         del locust_instance, tb
         self.__on_request_failure('', '', 0, exception)
 
-    def __on_worker_report(self, client_id, data):
+    def __on_worker_report(self, client_id, data, **args):
         if data['stats'] or data['errors']:
             for item in data['stats']:
                 self.num_requests -= item['num_requests']
@@ -102,34 +100,30 @@ class LocustStarter(object):
         self.locust_stop_time = time.time()
 
     def execute(self):
-        if os.getenv("WORKERS_LDJSON"):
-            fname = os.getenv("WORKERS_LDJSON")
-            is_csv = False
-        elif os.getenv("JTL"):
+        events.init.add_listener(self.__on_init)
+        events.user_error.add_listener(self.__on_exception)
+        events.worker_report.add_listener(self.__on_worker_report)
+        events.quitting.add_listener(self.__on_quit)
+
+        if os.getenv("JTL"):    # regular locust worker
             fname = os.getenv("JTL")
-            is_csv = True
+            self.fhd = open(fname, 'wt')
+            fieldnames = list(self.__getrec(None, None, None, None).keys())
+            dialect = guess_csv_dialect(",".join(fieldnames))
+            self.writer = csv.DictWriter(self.fhd, fieldnames=fieldnames, dialect=dialect)
+            self.writer.writeheader()
+            events.request.add_listener(self.__on_request_success)
+            events.request.add_listener(self.__on_request_failure)
+        elif os.getenv("WORKERS_LDJSON"):   # master of distributed mode
+            fname = os.getenv("WORKERS_LDJSON")
+            self.fhd = open(fname, 'wt')
+            self.writer = None
         else:
             raise ValueError("Please specify JTL or WORKERS_LDJSON environment variable")
 
-        with open(fname, 'wt') as self.fhd:
-            if is_csv:
-                fieldnames = list(self.__getrec(None, None, None, None).keys())
-                dialect = guess_csv_dialect(",".join(fieldnames))
-                self.writer = csv.DictWriter(self.fhd, fieldnames=fieldnames, dialect=dialect)
-                self.writer.writeheader()
-                self.fhd.flush()
-            else:
-                self.writer = None  # FIXME: bad code design, have zero object for it
+        main.main()
 
-            events.init.add_listener(self.__on_init)
-            events.request_success.add_listener(self.__on_request_success)
-            events.request_failure.add_listener(self.__on_request_failure)
-            events.user_error.add_listener(self.__on_exception)
-            events.worker_report.add_listener(self.__on_worker_report)
-            events.quitting.add_listener(self.__on_quit)
-
-            main.main()
-            self.fhd.flush()
+        self.fhd.close()
 
 
 if __name__ == '__main__':
